"use strict";
var fs = require('fs');
var path = require('path');
var program = require('commander');
var mkdirp = require('mkdirp');
var asyncblock = require('asyncblock');
var glob = require('glob');
var dtsgenerator = require('./index');
var pkg = require('../package.json');
program
    .version(pkg.version)
    .usage('[options] <file ... | file patterns using node-glob>')
    .option('-o, --out [file]', 'output d.ts filename.')
    .option('-p, --prefix [type prefix]', 'set the prefix of interface name. default is nothing.')
    .parse(process.argv);
var opts = program;
if (opts.args.length === 0) {
    readSchemasFromStdin(processGenerate);
}
else {
    readSchemasFromFiles(processGenerate);
}
function readSchemasFromStdin(callback) {
    var data = '';
    process.stdin.setEncoding('utf-8');
    process.stdin.on('readable', function () {
        var chunk;
        while (chunk = process.stdin.read()) {
            if (typeof chunk === 'string') {
                data += chunk;
            }
        }
    });
    process.stdin.on('end', function () {
        var schemas = JSON.parse(data);
        if (!Array.isArray(schemas)) {
            schemas = [schemas];
        }
        callback(null, schemas);
    });
}
function readSchemasFromFiles(callback) {
    asyncblock(function (flow) {
        flow.errorCallback = function (err) {
            callback(err, null);
        };
        opts.args.forEach(function (arg) {
            var files = glob.sync(arg);
            files.forEach(function (file) {
                fs.readFile(file, { encoding: 'utf-8' }, flow.add(file));
            });
        });
        var contents = flow.wait();
        callback(null, Object.keys(contents).map(function (key) { return contents[key]; }));
    });
}
function processGenerate(err, schemas) {
    if (err) {
        throw err;
    }
    var result = dtsgenerator(schemas, opts.prefix);
    if (opts.out) {
        mkdirp.sync(path.dirname(opts.out));
        fs.writeFileSync(opts.out, result, { encoding: 'utf-8' });
    }
    else {
        console.log(result);
    }
}
