"use strict";
var Process = (function () {
    function Process(typePrefix) {
        if (typePrefix === void 0) { typePrefix = ''; }
        this.typePrefix = typePrefix;
        this.indentChar = ' ';
        this.indentStep = 4;
        this.indent = 0;
        this.results = '';
        this.alreadlyIndentThisLine = false;
    }
    Process.prototype.output = function (str) {
        this.doIndent();
        this.results += str;
        return this;
    };
    Process.prototype.outputType = function (type, primitive) {
        if (primitive === void 0) { primitive = false; }
        if (this.typePrefix && !primitive) {
            this.output(this.typePrefix);
        }
        this.output(type);
        return this;
    };
    Process.prototype.outputKey = function (name, optional) {
        if (optional === void 0) { optional = false; }
        if (/[^0-9A-Za-z_$]/.test(name) || /^\d/.test(name)) {
            this.output('\"').output(name).output('\"');
        }
        else {
            this.output(name);
        }
        if (optional) {
            this.output('?');
        }
        return this;
    };
    Process.prototype.outputLine = function (str) {
        this.doIndent();
        if (str) {
            this.output(str);
        }
        this.output('\n');
        this.alreadlyIndentThisLine = false;
        return this;
    };
    Process.prototype.outputJSDoc = function (description, parameters) {
        var _this = this;
        if (parameters === void 0) { parameters = {}; }
        if (!description && Object.keys(parameters).length === 0) {
            return;
        }
        description = description || '';
        this.outputLine('/**');
        description.split('\n').forEach(function (line) {
            _this.output(' * ').outputLine(line);
        });
        Object.keys(parameters).forEach(function (parameterKey) {
            var parameter = parameters[parameterKey];
            _this.output(' * @params {');
            switch (parameter.type) {
                case 'string':
                    _this.output('string');
                    break;
                case 'integer':
                case 'number':
                    _this.output('number');
                    break;
                case 'boolean':
                    _this.output('boolean');
                    break;
                default:
                    console.error(parameter);
                    throw new Error('unknown type');
            }
            _this.output('} ').output(parameterKey).output(' ').outputLine(parameter.description);
        });
        this.outputLine(' */');
        return this;
    };
    Process.prototype.doIndent = function () {
        if (!this.alreadlyIndentThisLine) {
            var indent = this.getIndent();
            this.results += indent;
            this.alreadlyIndentThisLine = true;
        }
        return this;
    };
    Object.defineProperty(Process.prototype, "indentLevel", {
        get: function () {
            return this.indent;
        },
        enumerable: true,
        configurable: true
    });
    Process.prototype.increaseIndent = function () {
        this.indent++;
        return this;
    };
    Process.prototype.decreaseIndent = function () {
        this.indent--;
        return this;
    };
    Process.prototype.getIndent = function () {
        return this.repeatString(this.indent * this.indentStep, this.indentChar);
    };
    Process.prototype.repeatString = function (n, s) {
        var result = '';
        for (var i = 0; i < n; i++) {
            result += s;
        }
        return result;
    };
    Process.prototype.toDefinition = function () {
        return this.results;
    };
    return Process;
}());
module.exports = Process;
